from io import StringIO
from pathlib import Path

import click
from ruamel.yaml import YAML

from anylearn.cli.anylearn_cli_config import AnylearnCliConfig
from anylearn.cli.utils import (
    cmd_confirm_or_abort,
    check_config,
    check_connection,
    cmd_error,
    cmd_info,
    cmd_success,
    cmd_warning,
    get_cmd_command,
)
from anylearn.interfaces import QuotaGroup


@click.group("quota")
def commands():
    """
    Operate quotas configured in Anylearn remote.
    """
    pass


@commands.command()
@check_config()
@check_connection()
@get_cmd_command()
def ls():
    """
    List all available quotas.
    """
    quotas = QuotaGroup.get_list()
    s = []
    for q in quotas:
        available = q.available()
        s.append({
            'name': q.name,
            'id': q.id,
            'available': {
                k: f"{available[k]}/{q.capacity[k]}"
                for k in q.capacity.keys()
            }
        })
    yaml = YAML()
    with StringIO() as stream:
        yaml.dump(s, stream)
        s = stream.getvalue()
    cmd_info(msg=str(s))


@commands.command()
@click.argument('quota_name')
@check_config()
@check_connection()
@get_cmd_command()
def template(quota_name: str):
    quotas = QuotaGroup.get_list()
    try:
        q = next(q for q in quotas if q.name == quota_name)
    except StopIteration:
        cmd_error(msg=f"Failed to get QuotaGroup {quota_name}")
        raise click.Abort
    req = [{q.id: q.default}]
    path = Path(f"resource_request_{q.name}.template.yaml")
    path.touch(exist_ok=True)
    yaml = YAML()
    yaml.dump(req, path)
    cmd_success(msg=(
        f"Resource request in QuotaGroup {q.name} "
        f"is templated in file `./{path}`. "
        "Feel free to customize its filename and content "
        "in order to use it in `anyctl run train` as `--resource-yaml`."
    ))



import responses
from requests.exceptions import RequestException

from anylearn.config import init_sdk
from anylearn.interfaces.resource.resource import ResourceState
from anylearn.interfaces.resource.model import Model
from anylearn.utils.errors import AnyLearnException, AnyLearnMissingParamException
from tests.base_test_case import BaseTestCase

class TestModel(BaseTestCase):
    @responses.activate
    def test_list_model_ok(self):
        responses.add(responses.GET, url=self._url("model/list"),
                      json=[
                          {
                              'id': "MODE001",
                              'name': "TestModel1",
                              'description': "test",
                              'state': ResourceState.READY,
                              'public': False,
                              'upload_time': "2020-01-01 00:00",
                              'creator_id': "USER001",
                              'node_id': "NODE001",
                              'algorithm_id': "ALGO001",
                          },
                          {
                              'id': "MODE002",
                              'name': "TestModel2",
                              'description': "test",
                              'state': ResourceState.READY,
                              'public': True,
                              'upload_time': "2020-01-02 00:00",
                              'creator_id': "USER002",
                              'node_id': "NODE002",
                              'algorithm_id': "ALGO002",
                          },
                      ],
                      status=200)
        models = Model.get_list()
        self.assertIsInstance(models, list)
        self.assertEqual(len(models), 2)
        self.assertIsInstance(models[0], Model)
        self.assertIsInstance(models[1], Model)
        self.assertEqual(models[0].id, "MODE001")
        self.assertEqual(models[1].id, "MODE002")
        self.assertEqual(models[0].algorithm_id, "ALGO001")
        self.assertEqual(models[1].algorithm_id, "ALGO002")

    @responses.activate
    def test_list_model_ko_unknown_response(self):
        responses.add(responses.GET, url=self._url("model/list"),
                      json={'msg': "Unknown response"},
                      status=200)
        with self.assertRaises(AnyLearnException) as ctx:
            Model.get_list()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_get_model_detail_explicit_call_ok(self):
        responses.add(responses.GET, url=self._url("model/query?id=MODE001"),
                      match_querystring=True, json=[{
                          'id': "MODE001",
                          'name': "TestModel1",
                          'description': "test",
                          'state': ResourceState.READY,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "test.tar.gz",
                          'is_zipfile': 1,
                          'file': "USER001/models/MODE001/test.tar.gz",
                          'size': "250.41",
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                          'algorithm_id': "ALGO001",
                      }],
                      status=200)
        model = Model(id="MODE001")
        model.get_detail()
        self.assertEqual(model.filename, "test.tar.gz")
        self.assertTrue(model.is_zipfile)
        self.assertEqual(model.file_path, "USER001/models/MODE001/test.tar.gz")
        self.assertEqual(model.size, "250.41")
        self.assertEqual(model.creator_id, "USER001")
        self.assertEqual(model.node_id, "NODE001")
        self.assertEqual(model.algorithm_id, "ALGO001")

    @responses.activate
    def test_get_model_detail_no_explicit_call_ok(self):
        responses.add(responses.GET, url=self._url("model/query?id=MODE001"),
                      match_querystring=True, json=[{
                          'id': "MODE001",
                          'name': "TestModel1",
                          'description': "test",
                          'state': ResourceState.READY,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "test.tar.gz",
                          'is_zipfile': 1,
                          'file': "USER001/models/MODE001/test.tar.gz",
                          'size': "250.41",
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                          'algorithm_id': "ALGO001",
                      }],
                      status=200)
        model = Model(id="MODE001", load_detail=True)
        self.assertEqual(model.name, "TestModel1")
        self.assertEqual(model.filename, "test.tar.gz")
        self.assertTrue(model.is_zipfile)
        self.assertEqual(model.file_path, "USER001/models/MODE001/test.tar.gz")
        self.assertEqual(model.size, "250.41")
        self.assertEqual(model.creator_id, "USER001")
        self.assertEqual(model.node_id, "NODE001")
        self.assertEqual(model.algorithm_id, "ALGO001")

    @responses.activate
    def test_get_model_detail_ko_403(self):
        responses.add(responses.GET, url=self._url("model/query?id=MODE403"),
                      match_querystring=True, status=403)
        model = Model(id="MODE403")
        with self.assertRaises(RequestException) as ctx:
            model.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, RequestException)
        self.assertEqual(e.response.status_code, 403)

    @responses.activate
    def test_get_model_detail_ko_unknown_response(self):
        responses.add(responses.GET, url=self._url("model/query?id=MODE250"),
                      match_querystring=True, json={'msg': "Unknown response"},
                      status=200)
        model = Model(id="MODE250")
        with self.assertRaises(AnyLearnException) as ctx:
            model.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_get_model_detail_empty_list(self):
        responses.add(responses.GET, url=self._url("model/query?id=MODE250"),
                      match_querystring=True, json=[],
                      status=200)
        model = Model(id="MODE250")
        with self.assertRaises(AnyLearnException) as ctx:
            model.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_create_model_ok(self):
        responses.add(responses.POST, url=self._url("model/add"),
                      json={'data': "MODE001", 'message': "模型添加成功"},
                      status=200)
        model = Model(name="TestModel001", filename="test.tar.gz",
                      algorithm_id="ALGO001")
        result = model.save()
        self.assertTrue(result)
        self.assertEqual(model.id, "MODE001")

    @responses.activate
    def test_create_model_ko_empty_name_filename_algoid(self):
        model = Model()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            model.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "Model缺少必要字段：['name', 'algorithm_id']")

    @responses.activate
    def test_create_model_ko_unknown_response(self):
        responses.add(responses.POST, url=self._url("model/add"),
                      json={'msg': "Unknown response"}, status=201)
        model = Model(name="TestModel", filename="model250.tar.gz",
                      algorithm_id="ALGO250")
        with self.assertRaises(AnyLearnException) as ctx:
            model.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_update_model_ok(self):
        responses.add(responses.PUT, url=self._url("model/update"),
                      json={'data': "MODE001", 'message': "模型信息更新成功"},
                      status=200)
        model = Model(id="MODE001", name="TestModel1", description="test m",
                      state=ResourceState.READY,
                      public=True,
                      upload_time="2020-01-01 00:00")
        result = model.save()
        self.assertTrue(result)

    @responses.activate
    def test_update_model_ko_empty_name(self):
        model = Model(id="MODE250", name="")
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            model.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "Model缺少必要字段：['name']")

    @responses.activate
    def test_update_model_ko_unknown_response(self):
        responses.add(responses.PUT, url=self._url("model/update"),
                      json={'msg': "Unknown response"},
                      status=200)
        model = Model(id="MODE250", name="TestModel")
        with self.assertRaises(AnyLearnException) as ctx:
            model.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_delete_model_ok(self):
        responses.add(responses.DELETE,
                      url=self._url("model/delete?id=MODE001&force=0"),
                      match_querystring=True,
                      json={'data': "MODE001", 'message': "模型删除成功"},
                      status=200)
        model = Model(id="MODE001")
        result = model.delete()
        self.assertTrue(result)

    @responses.activate
    def test_delete_model_ko_unknown_response(self):
        responses.add(responses.DELETE,
                      url=self._url("model/delete?id=MODE250&force=0"),
                      match_querystring=True,
                      json={'msg': "Unknown response"},
                      status=204)
        model = Model(id="MODE250")
        with self.assertRaises(AnyLearnException) as ctx:
            model.delete()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")



from os import name
import click

from anylearn.cli.anylearn_cli_config import AnylearnCliConfig
from anylearn.cli.utils import (
    check_config,
    cmd_error,
    cmd_info,
    cmd_success,
    get_cmd_command,
)

@click.group("config")
def commands():
    """
    Get or set project's options.
    """
    pass


@commands.command()
@check_config()
@get_cmd_command()
def ls():
    """
    Print full config in current project.
    """
    config = AnylearnCliConfig.load()
    cmd_info(msg=str(config))


@commands.command()
@click.argument('config_key')
@check_config()
@get_cmd_command()
def get(config_key: str):
    """
    Get config by key.
    """
    config = AnylearnCliConfig.load()
    try:
        val = config.gets(config_key=config_key)
        cmd_info(msg=val)
    except Exception as e:
        cmd_error(msg=str(e))
        raise click.Abort


@commands.command()
@click.argument('config_key')
@click.argument('value')
@check_config()
@get_cmd_command()
def set(config_key: str, value):
    """
    Set config by key.
    """
    config = AnylearnCliConfig.load()
    try:
        config.sets(config_key=config_key, value=value)
        AnylearnCliConfig.update(config)
        cmd_success(msg="OK")
    except Exception as e:
        cmd_error(msg=str(e))
        raise click.Abort



import requests
from requests.adapters import HTTPAdapter
from requests.exceptions import HTTPError
from urllib3.util.retry import Retry

from anylearn.config import AnylearnConfig

url_base = lambda :AnylearnConfig.cluster_address + '/api'


class RequestRetrySession(object):
    def __init__(
        self,
        retries=3,
        backoff_factor=1,
        status_forcelist=[429, 500, 502, 503, 504],
        allowed_methods=["HEAD", "GET", "POST", "PUT", "PATCH", "DELETE", "OPTIONS", "TRACE"],
        session=None,
    ):
        self.session = session or requests.session()
        retry = Retry(
            total=retries,
            read=retries,
            connect=retries,
            backoff_factor=backoff_factor,
            status_forcelist=status_forcelist,
            allowed_methods=allowed_methods,
        )
        adapter = HTTPAdapter(max_retries=retry)
        self.session.mount('http://', adapter)
        self.session.mount('https://', adapter)

    def __enter__(self):
        return self.session

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.session.close()


def __intercept(**kwargs):
    if 'headers' not in kwargs:
        kwargs['headers'] = {}
    kwargs['headers']['Authorization'] = "Bearer %s" % AnylearnConfig.token
    return kwargs


def __raise_for_status(res: requests.Response):
    http_error_msg = ''
    if isinstance(res.reason, bytes):
        try:
            reason = res.reason.decode('utf-8')
        except UnicodeDecodeError:
            reason = res.reason.decode('iso-8859-1')
    else:
        reason = res.reason
    
    if isinstance(res.content, bytes):
        try:
            content = res.content.decode('utf-8')
        except UnicodeDecodeError:
            content = res.content.decode('iso-8859-1')
    else:
        content = res.content

    if 400 <= res.status_code < 500:
        http_error_msg = u'%s: "%s" for url: %s' % (res.status_code, content, res.url)

    elif 500 <= res.status_code < 600:
        http_error_msg = u'%s: "%s" for url: %s' % (res.status_code, reason, res.url)

    if http_error_msg:
        raise HTTPError(http_error_msg, response=res)


def get_with_token(*args, **kwargs):
    kwargs = __intercept(**kwargs)
    with RequestRetrySession() as sess:
        res = sess.get(*args, **kwargs)
        __raise_for_status(res)
        res.encoding = "utf-8"
        return res.json()


def post_with_token(*args, **kwargs):
    kwargs = __intercept(**kwargs)
    with RequestRetrySession() as sess:
        res = sess.post(*args, **kwargs)
        __raise_for_status(res)
        res.encoding = "utf-8"
        return res.json()


def post_with_secret_key(*args, secret_key, **kwargs):
    if 'headers' not in kwargs:
        kwargs['headers'] = {}
    kwargs['headers']['Secret-Key'] = secret_key
    with RequestRetrySession() as sess:
        res = sess.post(*args, **kwargs)
        __raise_for_status(res)
        res.encoding = "utf-8"
        return res.json()


def delete_with_token(*args, **kwargs):
    kwargs = __intercept(**kwargs)
    with RequestRetrySession() as sess:
        res = sess.delete(*args, **kwargs)
        __raise_for_status(res)
        res.encoding = "utf-8"
        return res.json()


def put_with_token(*args, **kwargs):
    kwargs = __intercept(**kwargs)
    with RequestRetrySession() as sess:
        res = sess.put(*args, **kwargs)
        __raise_for_status(res)
        res.encoding = "utf-8"
        return res.json()


def patch_with_token(*args, **kwargs):
    kwargs = __intercept(**kwargs)
    with RequestRetrySession() as sess:
        res = sess.patch(*args, **kwargs)
        __raise_for_status(res)
        res.encoding = "utf-8"
        return res.json()



from functools import wraps
import sys
import time
import os
from pathlib import Path

import click

from anylearn.config import init_sdk
from anylearn.cli.anylearn_cli_config import AnylearnCliConfig


def cmd_error(msg: str):
    update_history_record(f"                      [ERROR] {msg}")
    click.echo(click.style(f"[ERROR] {msg}", fg="red"))


def cmd_warning(msg: str):
    click.echo(click.style(f"[WARNING] {msg}", fg="yellow"))


def cmd_success(msg: str):
    update_history_record(f"                      [SUCCESS] {msg}")
    click.echo(click.style(f"[SUCCESS] {msg}", fg="green"))


def cmd_info(msg: str):
    click.echo(msg)


def cmd_confirm_or_abort():
    # No confirm = abort, so no need for any conditions
    click.confirm("Are you sure you want to proceed?", abort=True)


def check_config():
    def wrapper(fn):
        """

        :param fn: 

        """

        @wraps(fn)
        def decorator(*args, **kwargs):
            config_path = AnylearnCliConfig.get_config_path()
            if not config_path.exists():
                msg = "Anylearn project does not exist in current directory. "
                cmd_error(msg)
                raise click.Abort()
            return fn(*args, **kwargs)

        return decorator

    return wrapper


def do_login(show_success=False):
    config = AnylearnCliConfig.load()
    remote = {
        'host': (
            config.remote['host']
            if 'host' in config.remote and config.remote['host']
            else click.prompt(click.style("Anylearn host", fg="yellow"))
        ),
        'username': (
            config.remote['username']
            if 'username' in config.remote and config.remote['username']
            else click.prompt(click.style("Anylearn username", fg="yellow"))
        ),
        'password': (
            config.remote['password']
            if 'password' in config.remote and config.remote['password']
            else click.prompt(click.style("Anylearn password", fg="yellow"), hide_input=True)
        ),
    }
    try:
        init_sdk(remote['host'], remote['username'], remote['password'])
        config.remote = remote
        AnylearnCliConfig.update(config)
        if show_success:
            cmd_success(msg="Logged into Anylearn remote.")
    except:
        config.remote = {
            'host': None,
            'username': None,
            'password': None,
        }
        AnylearnCliConfig.update(config)
        cmd_error(msg="Failed to establish connection to Anylearn remote.")
        raise click.Abort()


def do_logout():
    config = AnylearnCliConfig.load()
    config.remote = {
        'host': None,
        'username': None,
        'password': None,
    }
    AnylearnCliConfig.update(config)
    cmd_success(msg="Logged out from Anylearn remote.")


def check_connection():
    def wrapper(fn):
        """

        :param fn: 

        """

        @wraps(fn)
        def decorator(*args, **kwargs):
            do_login()
            return fn(*args, **kwargs)

        return decorator

    return wrapper


def get_cmd_command():
    def wrapper(fn):
        """

        :param fn: 

        """

        @wraps(fn)
        def decorator(*args, **kwargs):
            cmd = sys.argv
            dt = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())
            command = f"[{dt}] anyctl"
            for i in range(len(cmd)):
                command += (f"{cmd[i]} ") if i else " "
            update_history_record(command=command)
            return fn(*args, **kwargs)

        return decorator

    return wrapper


def get_history_file_path():
    cwd = Path(os.getcwd())
    file_path = cwd / 'cmd_history.txt'
    file_path.touch(exist_ok=True)
    return file_path

def update_history_record(command: str):
    with open(get_history_file_path(), 'a') as f:
        f.write(f"\r{command}")


option_force = click.option(
    '-f', '--force',
    is_flag=True,
    default=False,
    help="Skip prompt and force actions."
)



from anylearn.applications import (
    report_intermediate_metric,
    report_final_metric,
    sync_algorithm,
    quick_train,
)
from anylearn.config import (
    AnyLearnAuthException,
    AnylearnConfig,
    init_sdk,
)



from datetime import datetime
from typing import Optional

from anylearn.interfaces.base import BaseObject
from anylearn.utils.api import url_base, get_with_token
from anylearn.utils.errors import (
    AnyLearnException,
    AnyLearnNotSupportedException
)


class ServiceRecordState:
    """
    服务状态标识：

    - 0(WAITING)表示等待执行
    - 1(RUNNING)表示执行中
    - 2(FINISHED)表示已完成
    - -1(FAILED)表示失败
    """
    WAITING = 0
    RUNNING = 1
    FINISHED = 2
    FAILED = -1


class ServiceRecord(BaseObject):
    """
    Attributes
    ----------
    id
        模型服务结果的唯一标识符，自动生成，由SERE+uuid1生成的编码中后28个有效位（小写字母和数字）组成
    service_id
        模型服务ID
    inference_data_file_id
        上传文件ID
    state
        服务记录运行状态
    create_time
        服务记录创建时间
    finish_time
        服务记录完成时间
    result
        服务推理结果
    error
        错误信息
    load_detail
        初始化时是否加载详情
    """

    def __init__(self,
                 id,
                 service_id: Optional[str]=None,
                 inference_data_file_id: Optional[str]=None,
                 state: Optional[int]=None,
                 create_time: Optional[datetime]=None,
                 finish_time: Optional[datetime]=None,
                 result: Optional[str]=None,
                 error: Optional[str]=None,
                 load_detail=False):
        """
        Parameters
        ----------
        id
            模型服务结果的唯一标识符，自动生成，由SERE+uuid1生成的编码中后28个有效位（小写字母和数字）组成
        service_id
            模型服务ID
        inference_data_file_id
            上传文件ID
        state
            服务记录运行状态
        create_time
            服务记录创建时间
        finish_time
            服务记录完成时间
        result
            服务推理结果
        error
            错误信息
        load_detail
            初始化时是否加载详情
        """
        self.service_id = service_id
        self.inference_data_file_id = inference_data_file_id
        self.state = state
        self.create_time = create_time
        self.finish_time = finish_time
        self.result = result
        self.error = error
        super().__init__(id=id, load_detail=load_detail)
    
    @classmethod
    def get_list(cls):
        """
        AnyLearnSDK接口不支持获取服务记录列表
        """
        raise AnyLearnNotSupportedException

    def get_detail(self):
        """
        获取服务记录详细信息

        - 对象属性 :obj:`id` 应为非空

        Returns
        -------
        ServiceRecord
            服务记录对象。
        """
        self._check_fields(required=['id'])
        res = get_with_token(f"{url_base()}/service_record/query",
                             params={ 'id': self.id })
        if not res or not isinstance(res, list):
            raise AnyLearnException("请求未能得到有效响应")
        res = res[0]
        self.__init__(id=res['id'], service_id=res['service_id'],
                      inference_data_file_id=res['file_id'],
                      state=res['state'],
                      create_time=res['create_time'],
                      finish_time=res['finish_time'],
                      result=res['result'],
                      error=res['error'])
    
    def save(self):
        """
        AnyLearnSDK接口不支持服务记录创建
        """
        raise AnyLearnNotSupportedException

    def delete(self):
        """
        AnyLearnSDK接口不支持服务记录删除
        """
        raise AnyLearnNotSupportedException

    def _namespace(self):
        return "service_record"



import aiohttp
from aioresponses import aioresponses

from anylearn.interfaces.resource.resource_uploader import AsyncResourceUploader
from tests.base_test_case import BaseTestCase

class TestResourceUploader(BaseTestCase):
    def test_run_async_resource_uploader_ok(self):
        n_chunks = 4

        n_total = n_progress = 0

        def on_start(total):
            nonlocal n_total
            n_total = total

        def on_progress(curr, _):
            nonlocal n_progress
            n_progress += 1

        uploader = AsyncResourceUploader(on_start=on_start,
                                         on_progress=on_progress)

        with aioresponses() as mock_responses:
            [mock_responses.post(url=self._url("resource/upload"),
                                 payload={
                                     'message': "OK",
                                 },
                                 status=200)
                for i in range(n_chunks)]
            res = uploader.run(resource_id="FILE001",
                               chunks=["test"] * n_chunks)
        self.assertTrue(res)
        self.assertEqual(n_total, n_chunks)
        self.assertEqual(n_progress, n_chunks)

    def test_run_resource_uploader_ko_500(self):
        n_chunks = 4
        with aioresponses() as mock_responses:
            uploader = AsyncResourceUploader()
            mock_responses.post(url=self._url("resource/upload"),
                                status=500)
            with self.assertRaises(aiohttp.ClientResponseError) as ctx:
                uploader.run(resource_id="FILE001",
                             chunks=["test"] * n_chunks)
            e = ctx.exception
            self.assertIsInstance(e, aiohttp.ClientResponseError)



from sqlalchemy import (
    Column,
    DateTime,
    Float,
    Integer,
    String,
    ForeignKey,
)
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship, backref
from sqlalchemy.sql.sqltypes import Float

from anylearn.utils import utc_plus_8


Base = declarative_base()

class SqlLocalAlgorithm(Base):
    __tablename__ = 'local_algorithm'

    id = Column(String(32), primary_key=True)
    checksum = Column(String(128), nullable=False)


class SqlLocalDataset(Base):
    __tablename__ = 'local_dataset'

    id = Column(String(32), primary_key=True)
    checksum = Column(String(128), nullable=False)


class SqlLocalModel(Base):
    __tablename__ = 'local_model'

    id = Column(String(32), primary_key=True)
    checksum = Column(String(128), nullable=False)


class SqlLocalHpoExperiment(Base):
    __tablename__ = 'local_hpo_experiment'

    id = Column(String(8), primary_key=True)
    port = Column(Integer, nullable=False)
    mode = Column(String(8), nullable=False, default="maximize")
    status = Column(String(16), nullable=False, default="RUNNING")
    project_id = Column(String(32), nullable=False)
    project_name = Column(String(255), nullable=False)
    algorithm_id = Column(String(32), nullable=False)
    algorithm_dir = Column(String(4096), nullable=True)
    algorithm_archive = Column(String(4096), nullable=True)
    dataset_id = Column(String(32), nullable=False)
    dataset_dir = Column(String(4096), nullable=True)
    dataset_archive = Column(String(4096), nullable=True)
    created_at = Column(DateTime, nullable=False, default=utc_plus_8())


class SqlLocalTrainTask(Base):
    __tablename__ = 'local_train_task'

    id = Column(String(32), primary_key=True)
    remote_state_sofar = Column(Integer, nullable=False)
    secret_key = Column(String(32), nullable=False)
    project_id = Column(String(32), nullable=False)
    hpo_id = Column(String(8), nullable=True)
    hpo_final_metric = Column(Float, nullable=True)

    def __repr__(self):
        return '<SqlLocalTrainTask %r %r>' % (self.id, self.remote_state_sofar)


class SqlTrainProfile(Base):
    __tablename__ = 'local_train_profile'

    id = Column(String(32), primary_key=True)
    train_task_id = Column(String(32), ForeignKey('local_train_task.id', ondelete="SET NULL"))
    train_task = relationship('SqlLocalTrainTask', backref=backref('train_profile', lazy="joined"), lazy="joined")
    algorithm_id = Column(String(32), nullable=False)
    algorithm_dir = Column(String(4096), nullable=True)
    algorithm_archive = Column(String(4096), nullable=True)
    dataset_id = Column(String(32), nullable=True)
    dataset_dir = Column(String(4096), nullable=True)
    dataset_archive = Column(String(4096), nullable=True)
    entrypoint = Column(String(4096), nullable=True)
    train_params = Column(String(4096), nullable=True)
    created_at = Column(DateTime, nullable=False, default=utc_plus_8())

    def __repr__(self):
        return '<SqlTrainProfile %r %r>' % (self.id, self.train_task_id)



from pathlib import Path
from typing import List
import responses
from unittest.mock import patch, create_autospec

from anylearn.applications.quickstart import quick_train, resume_unfinished_local_train_tasks
from anylearn.interfaces import Project, TrainTask, TrainTaskState
from anylearn.interfaces.resource import (
    Algorithm,
    Dataset,
    ResourceState,
)
from anylearn.utils.errors import (
    AnyLearnException,
    AnyLearnNotSupportedException,
)
from tests.base_test_case import BaseTestCase


FAKE_ALGO_OK = Path(__file__).parent / "fake_algo_ok"
FAKE_ALGO_KO = Path(__file__).parent / "fake_algo_ko"
FAKE_DSET = Path(__file__).parent / "fake_dset"


class TestQuickTrain(BaseTestCase):
    @patch('anylearn.interfaces.resource.resource_uploader.ResourceUploader')
    @responses.activate
    def test_quick_train_ok(self, MockResourceUploader):
        @create_autospec
        def mock_upload_ok(resource_id, chunks):
            return True

        # Mock uploader with stubbed upload function
        uploader = MockResourceUploader()
        uploader.run = mock_upload_ok

        # Stub mirror list
        responses.add(responses.GET, url=self._url("mirror/list"),
                      json=[
                          {
                              'id': "MIRR001",
                              'name': "Test001",
                              'requirements': "test==0.0.1",
                          },
                          {
                              'id': "MIRR002",
                              'name': "QUICKSTART",
                              'requirements': "test==0.0.2",
                          },
                      ],
                      status=200)
        # Stub algo creation
        responses.add(responses.POST, url=self._url("algorithm/add"),
                      json={'data': "ALGO001", 'message': "算法添加成功"},
                      status=200)
        # Stub dset creation
        responses.add(responses.POST, url=self._url("dataset/add"),
                      json={'data': "DSET001", 'message': "数据集添加成功"},
                      status=200)
        # Stub project creation
        responses.add(responses.POST, url=self._url("project/add"),
                      json={'data': "PROJ001", 'message': "项目添加成功"},
                      status=200)
        # Stub algo upload finish
        responses.add(responses.POST, url=self._url("resource/upload_finish"),
                      json={
                          'msg': "上传已完成，稍后可查看上传结果"
                      },
                      status=200)
        # Stub dset upload finish
        responses.add(responses.POST, url=self._url("resource/upload_finish"),
                      json={
                          'msg': "上传已完成，稍后可查看上传结果"
                      },
                      status=200)
        # Stub algo query
        responses.add(responses.GET, url=self._url("algorithm/query?id=ALGO001"),
                      match_querystring=True, json=[{
                          'id': "ALGO001",
                          'name': "Test-Algo_1.1",
                          'description': "test",
                          'state': ResourceState.READY,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "buzhongyao",
                          'is_zipfile': 1,
                          'file': "USER001/algos/ALGO001/buzhongyao",
                          'size': 250.41,
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                          'tags': "tag1, tag2",
                          'train_params': '[{"test": "test"}]',
                          'mirror_id': "MIRR002",
                          'follows_anylearn_norm': False,
                          'git_address': "http://anylearn.testing/anylearn/Test-Algo_1.1.git",
                          'git_migrated': True,
                      }],
                      status=200)
        # Stub dset query
        responses.add(responses.GET, url=self._url("dataset/query?id=DSET001"),
                      match_querystring=True, json=[{
                          'id': "DSET001",
                          'name': "TestDset1",
                          'description': "test",
                          'state': ResourceState.READY,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "buzhongyao",
                          'is_zipfile': 1,
                          'file': "USER001/datasets/DSET001/buzhongyao",
                          'size': 250.41,
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                      }],
                      status=200)
        # Stub train task creation
        responses.add(responses.POST, url=self._url("train_task/add"),
                      json={'data': "TRAI001", 'message': "服务添加成功"},
                      status=200)
        # Stub train task query
        responses.add(responses.GET, url=self._url("train_task/query?id=TRAI001"),
                      match_querystring=True, json=[{
                          'id': "TRAI001",
                          'name': "TestTrainTask1",
                          'description': "test",
                          'state': TrainTaskState.RUNNING,
                          'creator_id': "USER001",
                          'project_id': "PROJ001",
                          'algorithm_id': "ALGO001",
                          'algorithm_git_ref': 'master',
                          'args': "{\"arg1\":1, \"arg2\":\"test\"}",
                          'files': "DSET001",
                          'results_id': "FILE001",
                          'secret_key': "SECRET",
                          'create_time': "2021-03-01 23:59:59",
                          'finish_time': "",
                          'envs': "",
                          'resource_request': '[{"default": {"CPU": 1}}]',
                          'entrypoint': "python train.py",
                          'output': "output",
                          'mirror_id': "MIRR001",
                      }],
                      status=200)

        train_task, algo, dsets, project = quick_train(
            algorithm_dir=FAKE_ALGO_OK,
            algorithm_name="Test-Algo_1.1",
            dataset_dir=FAKE_DSET,
            entrypoint="python main.py",
            output="output",
            resource_uploader=uploader,
            resource_polling=0.1,
            dataset_hyperparam_name="dataset",
            hyperparams={'test_param': "test"}
        )
        
        self.assertIsInstance(train_task, TrainTask)
        self.assertEqual(train_task.id, "TRAI001")
        self.assertIsInstance(algo, Algorithm)
        self.assertEqual(algo.id, "ALGO001")
        self.assertEqual(algo.name, "Test-Algo_1.1")
        self.assertIsInstance(dsets, List)
        self.assertEqual(dsets[0].id, "DSET001")
        self.assertIsInstance(project, Project)
        self.assertEqual(project.id, "PROJ001")

    def test_quick_train_ko_algo_missing_requirements_txt(self):
        with self.assertRaises(AnyLearnException) as ctx:
            train_task = quick_train(algorithm_dir=FAKE_ALGO_KO,
                                     dataset_dir=FAKE_DSET,
                                     entrypoint="python main.py",
                                     output="output")
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "Missing 'requirements.txt' in algorithm directory")

    @responses.activate
    def test_quick_train_ko_algo_name_not_standard(self):
        # Stub mirror list
        responses.add(responses.GET, url=self._url("mirror/list"),
                      json=[
                          {
                              'id': "MIRR001",
                              'name': "Test001",
                              'requirements': "test==0.0.1",
                          },
                          {
                              'id': "MIRR002",
                              'name': "QUICKSTART",
                              'requirements': "test==0.0.2",
                          },
                      ],
                      status=200)
        with self.assertRaises(AnyLearnException) as ctx:
            train_task = quick_train(algorithm_dir=FAKE_ALGO_OK,
                                     algorithm_name="abc@123",
                                     dataset_dir=FAKE_DSET,
                                     entrypoint="python main.py",
                                     output="output")
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(
            e.msg,
            "Algorithm name should contain only alphanumeric, dash ('-'), underscore ('_') and dot ('.') characters"
        )

    @responses.activate
    def test_quick_train_ko_mirror_not_found(self):
        responses.add(responses.GET, url=self._url("mirror/list"),
                      json=[
                          {
                              'id': "MIRR001",
                              'name': "Test001",
                              'requirements': "test==0.0.1",
                          },
                          {
                              'id': "MIRR002",
                              'name': "Test002",
                              'requirements': "test==0.0.2",
                          },
                      ],
                      status=200)
        with self.assertRaises(AnyLearnNotSupportedException) as ctx:
            train_task = quick_train(algorithm_dir=FAKE_ALGO_OK,
                                     dataset_dir=FAKE_DSET,
                                     entrypoint="python main.py",
                                     output="output",
                                     mirror_name="QUICKSTART")
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnNotSupportedException)
        self.assertEqual(e.msg, "Container for `QUICKSTART` is not supported by the connected backend.")

    @patch('anylearn.interfaces.resource.resource_uploader.ResourceUploader')
    @responses.activate
    def test_quick_train_ko_algo_on_error(self, MockResourceUploader):
        @create_autospec
        def mock_upload_ok(resource_id, chunks):
            return True

        # Mock uploader with stubbed upload function
        uploader = MockResourceUploader()
        uploader.run = mock_upload_ok

        # Stub mirror list
        responses.add(responses.GET, url=self._url("mirror/list"),
                      json=[
                          {
                              'id': "MIRR001",
                              'name': "Test001",
                              'requirements': "test==0.0.1",
                          },
                          {
                              'id': "MIRR002",
                              'name': "QUICKSTART",
                              'requirements': "test==0.0.2",
                          },
                      ],
                      status=200)
        # Stub algo creation
        responses.add(responses.POST, url=self._url("algorithm/add"),
                      json={'data': "ALGO250", 'message': "算法添加成功"},
                      status=200)
        # Stub dset creation
        responses.add(responses.POST, url=self._url("dataset/add"),
                      json={'data': "DSET001", 'message': "数据集添加成功"},
                      status=200)
        # Stub project creation
        responses.add(responses.POST, url=self._url("project/add"),
                      json={'data': "PROJ001", 'message': "项目添加成功"},
                      status=200)
        # Stub algo upload finish
        responses.add(responses.POST, url=self._url("resource/upload_finish"),
                      json={
                          'msg': "上传已完成，稍后可查看上传结果"
                      },
                      status=200)
        # Stub dset upload finish
        responses.add(responses.POST, url=self._url("resource/upload_finish"),
                      json={
                          'msg': "上传已完成，稍后可查看上传结果"
                      },
                      status=200)
        # Stub algo query
        responses.add(responses.GET, url=self._url("algorithm/query?id=ALGO250"),
                      match_querystring=True, json=[{
                          'id': "ALGO250",
                          'name': "TestAlgo1",
                          'description': "test",
                          'state': ResourceState.ERROR,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "buzhongyao",
                          'is_zipfile': 1,
                          'file': "USER001/algos/ALGO250/buzhongyao",
                          'size': 250.41,
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                          'tags': "tag1, tag2",
                          'train_params': '[{"test": "test"}]',
                          'mirror_id': "MIRR002",
                          'follows_anylearn_norm': False,
                          'git_address': "http://anylearn.testing/anylearn/TestAlgo1.git",
                          'git_migrated': True,
                      }],
                      status=200)
        # Stub dset query
        responses.add(responses.GET, url=self._url("dataset/query?id=DSET001"),
                      match_querystring=True, json=[{
                          'id': "DSET001",
                          'name': "TestDset1",
                          'description': "test",
                          'state': ResourceState.READY,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "buzhongyao",
                          'is_zipfile': 1,
                          'file': "USER001/datasets/DSET001/buzhongyao",
                          'size': 250.41,
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                      }],
                      status=200)

        with self.assertRaises(AnyLearnException) as ctx:
            train_task = quick_train(algorithm_dir=FAKE_ALGO_OK,
                                     dataset_dir=FAKE_DSET,
                                     entrypoint="python main.py",
                                     output="output",
                                     resource_uploader=uploader,
                                     resource_polling=0.1)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "An error occured when uploading algorithm")

    @patch('anylearn.interfaces.resource.resource_uploader.ResourceUploader')
    @responses.activate
    def test_quick_train_ko_dset_on_error(self, MockResourceUploader):
        @create_autospec
        def mock_upload_ok(resource_id, chunks):
            return True

        # Mock uploader with stubbed upload function
        uploader = MockResourceUploader()
        uploader.run = mock_upload_ok

        # Stub mirror list
        responses.add(responses.GET, url=self._url("mirror/list"),
                      json=[
                          {
                              'id': "MIRR001",
                              'name': "Test001",
                              'requirements': "test==0.0.1",
                          },
                          {
                              'id': "MIRR002",
                              'name': "QUICKSTART",
                              'requirements': "test==0.0.2",
                          },
                      ],
                      status=200)
        # Stub algo creation
        responses.add(responses.POST, url=self._url("algorithm/add"),
                      json={'data': "ALGO001", 'message': "算法添加成功"},
                      status=200)
        # Stub dset creation
        responses.add(responses.POST, url=self._url("dataset/add"),
                      json={'data': "DSET250", 'message': "数据集添加成功"},
                      status=200)
        # Stub project creation
        responses.add(responses.POST, url=self._url("project/add"),
                      json={'data': "PROJ001", 'message': "项目添加成功"},
                      status=200)
        # Stub algo upload finish
        responses.add(responses.POST, url=self._url("resource/upload_finish"),
                      json={
                          'msg': "上传已完成，稍后可查看上传结果"
                      },
                      status=200)
        # Stub dset upload finish
        responses.add(responses.POST, url=self._url("resource/upload_finish"),
                      json={
                          'msg': "上传已完成，稍后可查看上传结果"
                      },
                      status=200)
        # Stub algo query
        responses.add(responses.GET, url=self._url("algorithm/query?id=ALGO001"),
                      match_querystring=True, json=[{
                          'id': "ALGO001",
                          'name': "TestAlgo1",
                          'description': "test",
                          'state': ResourceState.READY,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "buzhongyao",
                          'is_zipfile': 1,
                          'file': "USER001/algos/ALGO001/buzhongyao",
                          'size': 250.41,
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                          'tags': "tag1, tag2",
                          'train_params': '[{"test": "test"}]',
                          'mirror_id': "MIRR002",
                          'follows_anylearn_norm': False,
                          'git_address': "http://anylearn.testing/anylearn/TestAlgo1.git",
                          'git_migrated': True,
                      }],
                      status=200)
        # Stub dset query
        responses.add(responses.GET, url=self._url("dataset/query?id=DSET250"),
                      match_querystring=True, json=[{
                          'id': "DSET250",
                          'name': "TestDset1",
                          'description': "test",
                          'state': ResourceState.ERROR,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "buzhongyao",
                          'is_zipfile': 1,
                          'file': "USER001/datasets/DSET250/buzhongyao",
                          'size': 250.41,
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                      }],
                      status=200)

        with self.assertRaises(AnyLearnException) as ctx:
            train_task = quick_train(algorithm_dir=FAKE_ALGO_OK,
                                     dataset_dir=FAKE_DSET,
                                     entrypoint="python main.py",
                                     output="output",
                                     resource_uploader=uploader,
                                     resource_polling=0.1)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "Error occured when uploading dataset")

    @responses.activate
    def test_update_local_train_task(self):
        responses.add(responses.GET, url=self._url("train_task/query?id=TRAI000"),
                match_querystring=True, json=[{
                    'id': "TRAI000",
                    'name': "TestTrainTask1",
                    'description': "test",
                    'state': TrainTaskState.FAIL,
                    'creator_id': "USER001",
                    'project_id': "PROJ001",
                    'algorithm_id': "ALGO001",
                    'args': "{\"arg1\":1, \"arg2\":\"test\"}",
                    'files': "DSET001",
                    'results_id': "FILE001",
                    'secret_key': "SECRET",
                    'create_time': "2021-03-01 23:59:59",
                    'finish_time': "",
                    'envs': "",
                }],
                status=200)

        responses.add(responses.GET, url=self._url("train_task/query?id=TRAI001"),
                match_querystring=True, json=[{
                    'id': "TRAI001",
                    'name': "TestTrainTask1",
                    'description': "test",
                    'state': TrainTaskState.ABORT,
                    'creator_id': "USER001",
                    'project_id': "PROJ001",
                    'algorithm_id': "ALGO001",
                    'args': "{\"arg1\":1, \"arg2\":\"test\"}",
                    'files': "DSET001",
                    'results_id': "FILE001",
                    'secret_key': "SECRET",
                    'create_time': "2021-03-01 23:59:59",
                    'finish_time': "",
                    'envs': "",
                }],
                status=200)

        responses.add(responses.GET, url=self._url("train_task/query?id=TRAI002"),
                match_querystring=True, json=[{
                    'id': "TRAI002",
                    'name': "TestTrainTask1",
                    'description': "test",
                    'state': TrainTaskState.SUCCESS,
                    'creator_id': "USER001",
                    'project_id': "PROJ001",
                    'algorithm_id': "ALGO001",
                    'args': "{\"arg1\":1, \"arg2\":\"test\"}",
                    'files': "DSET001",
                    'results_id': "FILE001",
                    'secret_key': "SECRET",
                    'create_time': "2021-03-01 23:59:59",
                    'finish_time': "",
                    'envs': "",
                }],
                status=200)
 
        task_list = resume_unfinished_local_train_tasks()
        if len(task_list):
            self.assertIsInstance(task_list[0], TrainTask)
            self.assertEqual((task_list[0]).id, "TRAI001")
            self.assertEqual(task_list[0].state, -3)



import click

from anylearn.cli.anylearn_cli_config import AnylearnCliConfig
from anylearn.cli.utils import (
    cmd_confirm_or_abort,
    option_force,
    check_config,
    cmd_error,
    cmd_info,
    cmd_success,
    cmd_warning,
    get_cmd_command,
)


@click.group("rm")
def commands():
    """
    Remove of local algorithm or dataset from local Anylearn project.
    """
    pass


@commands.command()
@click.argument('name')
@option_force
@check_config()
@get_cmd_command()
def algorithm(name: str, force: bool=False):
    config = AnylearnCliConfig.load()
    try:
        algo = config.algorithms[name]
        dir = config.path['algorithm'][name]
    except KeyError:
        cmd_error(msg=(
            "Algorithm {name} or its path config does not exist.\n"
        ))
        raise click.Abort
    cmd_warning(msg=(
        "Algorithm to remove:"
        "\n(\n"
        f"  ID={algo.id},\n"
        f"  NAME={algo.name},\n"
        f"  DESCRIPTION={algo.description},\n"
        f"  LOCAL_DIR={dir},\n"
        "\n)\n"
        "Note: this action only removes the reference in current project. "
        "Local files and remote entries are kept as is."
    ))
    if not force:
        cmd_confirm_or_abort()
    del(config.algorithms[name])
    del(config.path['algorithm'][name])
    AnylearnCliConfig.update(config)
    cmd_success(msg="REMOVED")


@commands.command()
@click.argument('name')
@option_force
@check_config()
@get_cmd_command()
def dataset(name: str, force: bool=False):
    config = AnylearnCliConfig.load()
    try:
        dset = config.datasets[name]
        dir = config.path['dataset'][name]
    except KeyError:
        cmd_error(msg=(
            "Dataset {name} or its path config does not exist.\n"
        ))
        raise click.Abort
    cmd_warning(msg=(
        "Dataset to remove:"
        "\n(\n"
        f"  ID={dset.id},\n"
        f"  NAME={dset.name},\n"
        f"  DESCRIPTION={dset.description},\n"
        f"  LOCAL_DIR={dir},\n"
        "\n)\n"
        "Note: this action only removes the reference in current project. "
        "Local files and remote entries are kept as is."
    ))
    if not force:
        cmd_confirm_or_abort()
    del(config.datasets[name])
    del(config.path['dataset'][name])
    AnylearnCliConfig.update(config)
    cmd_success(msg="REMOVED")



from typing import Optional
from abc import ABC, abstractclassmethod, abstractmethod

from anylearn.utils.api import (
    url_base,
    post_with_token,
    put_with_token,
    delete_with_token
)
from anylearn.utils.errors import (
    AnyLearnException,
    AnyLearnMissingParamException
)


class BaseObject(ABC):

    """具体资源信息配置"""
    _fields = {
        # 资源创建/更新请求包体中必须包含且不能为空的字段
        'required': {
            'create': [],
            'update': [],
        },
        # 资源创建/更新请求包体中包含的所有字段
        'payload': {
            'create': [],
            'update': [],
        },
    }
    """
    所有子类中需定制 :obj:`_fields` 的字段内容以满足创建/更新所需要的字段

    创建/更新对象时：

    - 必须包含且不能为空的字段 :obj:`_fields['required']`
    - 所有字段 :obj:`_fields['payload']`
    """

    def __init__(self,
                 id: Optional[str]=None,
                 load_detail=False):
        """
        Parameters
        ----------
        id
            对象的ID
        load_detail
            初始化对象时是否加载详情
        """
        self.id = id
        if load_detail:
            self.get_detail()

    @abstractclassmethod
    def get_list(self) -> list:
        """
        获取对象列表，子类需实现此抽象方法
        """
        raise NotImplementedError

    @abstractmethod
    def get_detail(self):
        """
        获取对象详情，子类需实现此抽象方法
        """
        raise NotImplementedError

    @abstractmethod
    def _namespace(self):
        """
        - 子类的命名空间，调用此方法以获取子类的名称用于异常信息输出等，以 :obj:`User` 为例， :obj:`_namespace` 可以为 :obj:`user` 
        - 子类需实现此抽象方法
        """
        raise NotImplementedError

    def save(self):
        """
        创建或更新对象

        - 对象包含非空属性 :obj:`id` 时为更新，否则为创建
        - 创建对象时必须包含且不能为空的字段： :obj:`_fields['required']['create']`
        - 创建对象时包含的所有字段： :obj:`_fields['payload']['create']`
        - 更新对象时必须包含且不能为空的字段： :obj:`_fields['required']['update']`
        - 更新对象时包含的所有字段： :obj:`_fields['payload']['update']`
            
        Returns
        -------
        bool
            True or False
        """
        if self.id:
            self._check_fields(required=self._fields['required']['update'])
            return self._update()
        else:
            self._check_fields(required=self._fields['required']['create'])
            return self._create()

    def _update(self):
        """
        更新对象，如果子类更新方法与此有较大差异可以重写此方法
        """
        data = self._payload_update()
        res = put_with_token(self._url_update(), data=data)
        if not res or 'data' not in res:
            raise AnyLearnException("请求未能得到有效响应")
        return self.id == res['data']

    def _payload_update(self):
        return {k: self.__getattribute__(k)
                for k in self._fields['payload']['update']}

    def _create(self):
        """
        创建对象，如果子类创建方法与此有较大差异可以重写此方法
        """
        data = self._payload_create()
        res = post_with_token(self._url_create(), data=data)
        if not res or 'data' not in res:
            raise AnyLearnException("请求未能得到有效响应")
        self.id = res['data']
        return True

    def _payload_create(self):
        return {k: self.__getattribute__(k)
                for k in self._fields['payload']['create']}

    def delete(self, force: bool=False):
        """
        删除对象

        - 对象属性 :obj:`id` 应为非空

        Returns
        -------
        bool
            True or False
        """
        self._check_fields(required=['id'])
        res = delete_with_token(self._url_delete(),
                                params={
                                    'id': self.id,
                                    'force': 1 if force else 0,
                                })
        if not res or 'data' not in res:
            raise AnyLearnException("请求未能得到有效响应")
        return self.id == res['data']

    def _check_fields(self, required=[]):
        """
        对象检查属性是否存在
        """
        missed = [field
                  for field in required
                  if not self.__getattribute__(field)]
        if missed:
            msg = f"{self.__class__.__name__}缺少必要字段：{missed}"
            raise AnyLearnMissingParamException(msg)

    def _url_create(self):
        """
        创建对象url，如果子类创建对象接口名称不是 :obj:`add` ，可以重写此方法来定制接口名称
        """
        return f"{url_base()}/{self._namespace()}/add"

    def _url_update(self):
        """
        更新对象url，如果子类更新对象接口名称不是 :obj:`update` ，可以重写此方法来定制接口名称
        """
        return f"{url_base()}/{self._namespace()}/update"

    def _url_delete(self):
        """
        删除对象url，如果子类删除对象接口名称不是 :obj:`delete` ，可以重写此方法来定制接口名称
        """
        return f"{url_base()}/{self._namespace()}/delete"

    def __repr__(self):
        d = self.__dict__
        kv = ", ".join([f"{k}={d[k]!r}" for k in d])
        return f"{self.__class__.__name__}({kv})"



import responses
from requests.exceptions import RequestException

from anylearn.interfaces.project import Project
from anylearn.utils.errors import AnyLearnException, AnyLearnMissingParamException
from tests.base_test_case import BaseTestCase

class TestProject(BaseTestCase):
    @responses.activate
    def test_list_project_ok(self):
        responses.add(responses.GET, url=self._url("project/list"),
                      json=[
                          {
                              'id': "PROJ001",
                              'name': "TestPROJ1",
                              'description': "test",
                              'create_time': "2020-03-01 00:00",
                              'update_time': "2020-03-01 00:00",
                          },
                          {
                              'id': "PROJ002",
                              'name': "TestPROJ2",
                              'description': "test",
                              'create_time': "2020-03-02 00:00",
                              'update_time': "2020-03-02 00:00",
                          },
                      ],
                      status=200)
        projects = Project.get_list()
        self.assertIsInstance(projects, list)
        self.assertEqual(len(projects), 2)
        self.assertIsInstance(projects[0], Project)
        self.assertIsInstance(projects[1], Project)
        self.assertEqual(projects[0].id, "PROJ001")
        self.assertEqual(projects[1].id, "PROJ002")

    @responses.activate
    def test_list_project_ko_unknown_response(self):
        responses.add(responses.GET, url=self._url("project/list"),
                      json={'msg': "Unknown response"},
                      status=200)
        with self.assertRaises(AnyLearnException) as ctx:
            Project.get_list()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_get_project_detail_explicit_call_ok(self):
        responses.add(responses.GET, url=self._url("project/query?id=PROJ001"),
                      match_querystring=True, json=[{
                          'id': "PROJ001",
                          'name': "TestProj1",
                          'description': "test",
                          'create_time': "2020-03-01 00:00",
                          'update_time': "2020-03-01 00:00",
                          'creator_id': "USER001",
                          'datasets': ["DSET001", "DSET002"]
                      }],
                      status=200)
        project = Project(id="PROJ001")
        project.get_detail()
        self.assertEqual(project.name, "TestProj1")
        self.assertEqual(project.description, "test")
        self.assertEqual(project.create_time, "2020-03-01 00:00")
        self.assertEqual(project.update_time, "2020-03-01 00:00")
        self.assertEqual(project.creator_id, "USER001")
        self.assertIsInstance(project.datasets, list)
        if isinstance(project.datasets, list):
            self.assertEqual(len(project.datasets), 2)
        self.assertEqual(project.datasets[0], "DSET001")
        self.assertEqual(project.datasets[1], "DSET002")

    @responses.activate
    def test_get_project_detail_no_explicit_call_ok(self):
        responses.add(responses.GET, url=self._url("project/query?id=PROJ002"),
                      match_querystring=True, json=[{
                          'id': "PROJ002",
                          'name': "TestProj2",
                          'description': "test",
                          'create_time': "2020-03-02 00:00",
                          'update_time': "2020-03-02 00:00",
                          'creator_id': "USER001",
                          'datasets': ["DSET001", "DSET002"]
                      }],
                      status=200)
        project = Project(id="PROJ002")
        project.get_detail()
        self.assertEqual(project.name, "TestProj2")
        self.assertEqual(project.description, "test")
        self.assertEqual(project.create_time, "2020-03-02 00:00")
        self.assertEqual(project.update_time, "2020-03-02 00:00")
        self.assertEqual(project.creator_id, "USER001")
        self.assertIsInstance(project.datasets, list)
        if isinstance(project.datasets,list):
            self.assertEqual(len(project.datasets), 2)
        self.assertEqual(project.datasets[0], "DSET001")
        self.assertEqual(project.datasets[1], "DSET002")

    @responses.activate
    def test_get_project_detail_ko_403(self):
        responses.add(responses.GET, url=self._url("project/query?id=PROJ403"),
                      match_querystring=True, status=403)
        project = Project(id="PROJ403")
        with self.assertRaises(RequestException) as ctx:
            project.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, RequestException)
        self.assertEqual(e.response.status_code, 403)

    @responses.activate
    def test_get_project_detail_ko_unknown_response(self):
        responses.add(responses.GET, url=self._url("project/query?id=DSET250"),
                      match_querystring=True, json={'msg': "Unknown response"},
                      status=200)
        project = Project(id="DSET250")
        with self.assertRaises(AnyLearnException) as ctx:
            project.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_get_project_detail_empty_list(self):
        responses.add(responses.GET, url=self._url("project/query?id=DSET250"),
                      match_querystring=True, json=[],
                      status=200)
        project = Project(id="DSET250")
        with self.assertRaises(AnyLearnException) as ctx:
            project.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_create_project_ok(self):
        responses.add(responses.POST, url=self._url("project/add"),
                      json={'data': "PROJ001", 'message': "项目添加成功"},
                      status=200)
        project = Project(name="TestProj001")
        result = project.save()
        self.assertTrue(result)
        self.assertEqual(project.id, "PROJ001")

    @responses.activate
    def test_create_project_ko_empty_name(self):
        project = Project()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            project.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "Project缺少必要字段：['name']")

    @responses.activate
    def test_create_project_ko_unknown_response(self):
        responses.add(responses.POST, url=self._url("project/add"),
                      json={'msg': "Unknown response"}, status=201)
        project = Project(name="TestProj250")
        with self.assertRaises(AnyLearnException) as ctx:
            project.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_update_project_ok(self):
        responses.add(responses.PUT, url=self._url("project/update"),
                      json={'data': "PROJ001", 'message': "项目信息更新成功"},
                      status=200)
        project = Project(id="PROJ001", name="TestProj1", description="test m")
        result = project.save()
        self.assertTrue(result)

    @responses.activate
    def test_update_project_ko_empty_name(self):
        project = Project(id="PROJ250", name="")
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            project.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "Project缺少必要字段：['name']")

    @responses.activate
    def test_update_project_ko_unknown_response(self):
        responses.add(responses.PUT, url=self._url("project/update"),
                      json={'msg': "Unknown response"},
                      status=200)
        project = Project(id="DSET250", name="TestDset", description="test")
        with self.assertRaises(AnyLearnException) as ctx:
            project.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_delete_project_ok(self):
        responses.add(responses.DELETE,
                      url=self._url("project/delete?id=PROJ001&force=0"),
                      match_querystring=True,
                      json={'data': "PROJ001", 'message': "项目删除成功"},
                      status=200)
        project = Project(id="PROJ001")
        result = project.delete()
        self.assertTrue(result)

    @responses.activate
    def test_delete_project_ko_unknown_response(self):
        responses.add(responses.DELETE,
                      url=self._url("project/delete?id=DSET250&force=0"),
                      match_querystring=True,
                      json={'msg': "Unknown response"},
                      status=204)
        project = Project(id="DSET250")
        with self.assertRaises(AnyLearnException) as ctx:
            project.delete()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")



from anylearn.storage.db.db import DB
from anylearn.storage.db.models import SqlLocalTrainTask
from anylearn.interfaces.train_task import TrainTask
from tests.base_test_case import BaseTestCase


class TestDB(BaseTestCase):

    def create_train_task(self):
        db = DB()
        train_task0 = TrainTask(id="TRAI000", project_id="pro000",
                                state=0, secret_key="000sk")
        train_task1 = TrainTask(id="TRAI001", project_id="pro111",
                                state=1, secret_key="111sk")
        train_task2 = TrainTask(id="TRAI002", project_id="pro222",
                                state=2, secret_key="222sk")
        db.create_or_update_train_task(train_task=train_task0)
        db.create_or_update_train_task(train_task=train_task1)
        db.create_or_update_train_task(train_task=train_task2)

    def test_train_task_resume(self):
        self.create_train_task()

        train_task_lsit = DB().get_unfinished_train_tasks()
        if len(train_task_lsit):
            self.assertIsInstance(train_task_lsit[0], SqlLocalTrainTask)
            self.assertEqual((train_task_lsit[0]).id, "TRAI000")
            self.assertEqual(train_task_lsit[0].remote_state_sofar, 0)



from .base import BaseObject
from .mirror import Mirror
from .project import Project
from .quota import QuotaGroup
from .resource import (
    Algorithm,
    AsyncResourceDownloader,
    AsyncResourceUploader,
    SyncResourceUploader,
    Dataset,
    Model,
    Resource,
    ResourceDownloader,
    ResourceState,
    ResourceUploader,
)
from .service import (
    Service,
    ServiceRecord,
    ServiceRecordState,
    ServiceState,
    ServiceVisibility,
)
from .train_task import TrainTask, TrainTaskState
from .user import User



import responses
from requests.exceptions import RequestException

from anylearn.config import init_sdk
from anylearn.interfaces.resource.resource import ResourceState
from anylearn.interfaces.resource.dataset import Dataset
from anylearn.utils.errors import AnyLearnException, AnyLearnMissingParamException
from tests.base_test_case import BaseTestCase

class TestDataset(BaseTestCase):
    @responses.activate
    def test_list_dataset_ok(self):
        responses.add(responses.GET, url=self._url("dataset/list"),
                      json=[
                          {
                              'id': "DSET001",
                              'name': "TestDset1",
                              'description': "test",
                              'state': ResourceState.READY,
                              'public': False,
                              'upload_time': "2020-01-01 00:00",
                          },
                          {
                              'id': "DSET002",
                              'name': "TestDset2",
                              'description': "test",
                              'state': ResourceState.READY,
                              'public': True,
                              'upload_time': "2020-01-02 00:00",
                          },
                      ],
                      status=200)
        datasets = Dataset.get_list()
        self.assertIsInstance(datasets, list)
        self.assertEqual(len(datasets), 2)
        self.assertIsInstance(datasets[0], Dataset)
        self.assertIsInstance(datasets[1], Dataset)
        self.assertEqual(datasets[0].id, "DSET001")
        self.assertEqual(datasets[1].id, "DSET002")

    @responses.activate
    def test_list_dataset_ko_unknown_response(self):
        responses.add(responses.GET, url=self._url("dataset/list"),
                      json={'msg': "Unknown response"},
                      status=200)
        with self.assertRaises(AnyLearnException) as ctx:
            Dataset.get_list()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_get_dataset_detail_explicit_call_ok(self):
        responses.add(responses.GET, url=self._url("dataset/query?id=DSET001"),
                      match_querystring=True, json=[{
                          'id': "DSET001",
                          'name': "TestDset1",
                          'description': "test",
                          'state': ResourceState.READY,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "test.tar.gz",
                          'is_zipfile': 1,
                          'file': "USER001/datasets/DSET001/test.tar.gz",
                          'size': "250.41",
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                      }],
                      status=200)
        dataset = Dataset(id="DSET001", name="TestDset1", description="test",
                          state=ResourceState.READY,
                          public=True,
                          upload_time="2020-01-01 00:00")
        dataset.get_detail()
        self.assertEqual(dataset.filename, "test.tar.gz")
        self.assertTrue(dataset.is_zipfile)
        self.assertEqual(dataset.file_path, "USER001/datasets/DSET001/test.tar.gz")
        self.assertEqual(dataset.size, "250.41")
        self.assertEqual(dataset.creator_id, "USER001")
        self.assertEqual(dataset.node_id, "NODE001")


    @responses.activate
    def test_get_dataset_detail_no_explicit_call_ok(self):
        responses.add(responses.GET, url=self._url("dataset/query?id=DSET001"),
                      match_querystring=True, json=[{
                          'id': "DSET001",
                          'name': "TestDset1",
                          'description': "test",
                          'state': ResourceState.READY,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "test.tar.gz",
                          'is_zipfile': 1,
                          'file': "USER001/datasets/DSET001/test.tar.gz",
                          'size': "250.41",
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                      }],
                      status=200)
        dataset = Dataset(id="DSET001", load_detail=True)
        self.assertEqual(dataset.name, "TestDset1")
        self.assertEqual(dataset.filename, "test.tar.gz")
        self.assertTrue(dataset.is_zipfile)
        self.assertEqual(dataset.file_path, "USER001/datasets/DSET001/test.tar.gz")
        self.assertEqual(dataset.size, "250.41")
        self.assertEqual(dataset.creator_id, "USER001")
        self.assertEqual(dataset.node_id, "NODE001")

    @responses.activate
    def test_get_dataset_detail_ko_403(self):
        responses.add(responses.GET, url=self._url("dataset/query?id=DSET403"),
                      match_querystring=True, status=403)
        dataset = Dataset(id="DSET403", name="TestDsetForbidden",
                          description="test",
                          state=ResourceState.READY,
                          public=False,
                          upload_time="2020-04-03 04:03")
        with self.assertRaises(RequestException) as ctx:
            dataset.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, RequestException)
        self.assertEqual(e.response.status_code, 403)

    @responses.activate
    def test_get_dataset_detail_ko_unknown_response(self):
        responses.add(responses.GET, url=self._url("dataset/query?id=DSET250"),
                      match_querystring=True, json={'msg': "Unknown response"},
                      status=200)
        dataset = Dataset(id="DSET250", name="TestDset", description="test",
                          state=ResourceState.READY,
                          public=False,
                          upload_time="2020-01-01 02:50")
        with self.assertRaises(AnyLearnException) as ctx:
            dataset.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_get_dataset_detail_empty_list(self):
        responses.add(responses.GET, url=self._url("dataset/query?id=DSET250"),
                      match_querystring=True, json=[],
                      status=200)
        dataset = Dataset(id="DSET250", name="TestDset", description="test",
                          state=ResourceState.READY,
                          public=False,
                          upload_time="2020-01-01 02:50")
        with self.assertRaises(AnyLearnException) as ctx:
            dataset.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_create_dataset_ok(self):
        responses.add(responses.POST, url=self._url("dataset/add"),
                      json={'data': "DSET001", 'message': "数据集添加成功"},
                      status=200)
        dataset = Dataset(name="TestDset001", filename="test.tar.gz")
        result = dataset.save()
        self.assertTrue(result)
        self.assertEqual(dataset.id, "DSET001")

    @responses.activate
    def test_create_dataset_ko_empty_name(self):
        dataset = Dataset()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            dataset.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "Dataset缺少必要字段：['name']")

    @responses.activate
    def test_create_dataset_ko_unknown_response(self):
        responses.add(responses.POST, url=self._url("dataset/add"),
                      json={'msg': "Unknown response"}, status=201)
        dataset = Dataset(name="TestDset", filename="dset250.tar.gz")
        with self.assertRaises(AnyLearnException) as ctx:
            dataset.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_update_dataset_ok(self):
        responses.add(responses.PUT, url=self._url("dataset/update"),
                      json={'data': "DSET001", 'message': "数据集信息更新成功"},
                      status=200)
        dataset = Dataset(id="DSET001", name="TestDset1", description="test m",
                          state=ResourceState.READY,
                          public=True,
                          upload_time="2020-01-01 00:00")
        result = dataset.save()
        self.assertTrue(result)

    @responses.activate
    def test_update_dataset_ko_empty_name(self):
        dataset = Dataset(id="DSET250", name="")
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            dataset.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "Dataset缺少必要字段：['name']")

    @responses.activate
    def test_update_dataset_ko_unknown_response(self):
        responses.add(responses.PUT, url=self._url("dataset/update"),
                      json={'msg': "Unknown response"},
                      status=200)
        dataset = Dataset(id="DSET250", name="TestDset", description="test",
                          state=ResourceState.READY,
                          public=False,
                          upload_time="2020-01-01 02:50")
        with self.assertRaises(AnyLearnException) as ctx:
            dataset.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_delete_dataset_ok(self):
        responses.add(responses.DELETE,
                      url=self._url("dataset/delete?id=DSET001&force=0"),
                      match_querystring=True,
                      json={'data': "DSET001", 'message': "数据集删除成功"},
                      status=200)
        dataset = Dataset(id="DSET001", name="TestDset1", description="test d",
                          state=ResourceState.READY,
                          public=True,
                          upload_time="2020-01-01 00:00")
        result = dataset.delete()
        self.assertTrue(result)

    @responses.activate
    def test_delete_dataset_ko_unknown_response(self):
        responses.add(responses.DELETE,
                      url=self._url("dataset/delete?id=DSET250&force=0"),
                      match_querystring=True,
                      json={'msg': "Unknown response"},
                      status=204)
        dataset = Dataset(id="DSET250", name="TestDset", description="test",
                          state=ResourceState.READY,
                          public=False,
                          upload_time="2020-01-01 02:50")
        with self.assertRaises(AnyLearnException) as ctx:
            dataset.delete()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")



from datetime import datetime, timezone, timedelta
import json
import uuid

from requests.exceptions import RequestException
import responses
from urllib.parse import urlencode
from unittest.mock import patch, create_autospec

from anylearn.interfaces.resource import Model, ResourceState
from anylearn.interfaces.train_task import TrainTask, TrainTaskState
from anylearn.utils.errors import AnyLearnException, AnyLearnMissingParamException
from tests.base_test_case import BaseTestCase
from anylearn.storage.db.db import DB
from anylearn.applications.train_profile import TrainProfile

class TestTrainTask(BaseTestCase):
    @responses.activate
    def test_get_train_task_detail_explicit_call_ok(self):
        responses.add(responses.GET, url=self._url("train_task/query?id=TRAI001"),
                      match_querystring=True, json=[{
                          'id': "TRAI001",
                          'name': "TestTrainTask1",
                          'description': "test",
                          'state': TrainTaskState.RUNNING,
                          'creator_id': "USER001",
                          'project_id': "PROJ001",
                          'algorithm_id': "ALGO001",
                          'algorithm_git_ref': None,
                          'args': "{\"arg1\":1, \"arg2\":\"test\"}",
                          'files': "DSET001",
                          'results_id': "FILE001",
                          'secret_key': "SECRET",
                          'create_time': "2021-03-01 23:59:59",
                          'finish_time': "",
                          'envs': "",
                          'resource_request': '[{"default": {"CPU": 1}}]',
                          'entrypoint': "python train.py",
                          'output': "output",
                          'mirror_id': "MIRR001",
                      }],
                      status=200)
        train_task = TrainTask(id="TRAI001")
        train_task.get_detail()
        self.assertEqual(train_task.name, "TestTrainTask1")
        self.assertEqual(train_task.description, "test")
        self.assertEqual(train_task.state, TrainTaskState.RUNNING)
        self.assertEqual(train_task.creator_id, "USER001")
        self.assertEqual(train_task.project_id, "PROJ001")
        self.assertEqual(train_task.algorithm_id, "ALGO001")
        if train_task.train_params:
            self.assertIsInstance(json.loads(train_task.train_params), dict)
        self.assertEqual(train_task.files, "DSET001")
        self.assertEqual(train_task.results_id, "FILE001")
        self.assertEqual(train_task.secret_key, "SECRET")
        self.assertEqual(train_task.create_time, "2021-03-01 23:59:59")

    @responses.activate
    def test_get_train_task_detail_no_explicit_call_ok(self):
        responses.add(responses.GET, url=self._url("train_task/query?id=TRAI002"),
                      match_querystring=True, json=[{
                          'id': "TRAI002",
                          'name': "TestTrainTask2",
                          'description': "test",
                          'state': TrainTaskState.RUNNING,
                          'creator_id': "USER002",
                          'project_id': "PROJ002",
                          'algorithm_id': "ALGO002",
                          'algorithm_git_ref': None,
                          'args': "{\"arg1\":1, \"arg2\":\"test\"}",
                          'files': "DSET001",
                          'results_id': "FILE002",
                          'secret_key': "SECRET",
                          'create_time': "2021-03-01 23:59:59",
                          'finish_time': "",
                          'envs': "",
                          'resource_request': '[{"default": {"CPU": 1}}]',
                          'entrypoint': "python train.py",
                          'output': "output",
                          'mirror_id': "MIRR002",
                      }],
                      status=200)
        train_task = TrainTask(id="TRAI002", load_detail=True)
        self.assertEqual(train_task.name, "TestTrainTask2")
        self.assertEqual(train_task.description, "test")
        self.assertEqual(train_task.state, TrainTaskState.RUNNING)
        self.assertEqual(train_task.creator_id, "USER002")
        self.assertEqual(train_task.project_id, "PROJ002")
        self.assertEqual(train_task.algorithm_id, "ALGO002")
        if train_task.train_params:
            self.assertIsInstance(json.loads(train_task.train_params), dict)
        self.assertEqual(train_task.files, "DSET001")
        self.assertEqual(train_task.results_id, "FILE002")
        self.assertEqual(train_task.secret_key, "SECRET")
        self.assertEqual(train_task.create_time, "2021-03-01 23:59:59")

    @responses.activate
    def test_get_train_task_detail_ko_403(self):
        responses.add(responses.GET, url=self._url("train_task/query?id=TRAI403"),
                      match_querystring=True, status=403)
        train_task = TrainTask(id="TRAI403")
        with self.assertRaises(RequestException) as ctx:
            train_task.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, RequestException)
        self.assertEqual(e.response.status_code, 403)

    @responses.activate
    def test_get_train_task_detail_ko_unknown_response(self):
        responses.add(responses.GET, url=self._url("train_task/query?id=TRAI250"),
                      match_querystring=True, json={'msg': "Unknown response"},
                      status=200)
        train_task = TrainTask(id="TRAI250")
        with self.assertRaises(AnyLearnException) as ctx:
            train_task.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_get_train_task_detail_empty_list(self):
        responses.add(responses.GET, url=self._url("train_task/query?id=TRAI250"),
                      match_querystring=True, json=[],
                      status=200)
        train_task = TrainTask(id="TRAI250")
        with self.assertRaises(AnyLearnException) as ctx:
            train_task.get_detail()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_create_train_task_ok(self):
        responses.add(responses.POST, url=self._url("train_task/add"),
                      json={'data': "TRAI001", 'message': "服务添加成功"},
                      status=200)
        train_task = TrainTask(name="TestTrainTask001", project_id="PROJ001",
                               algorithm_id="ALGO001",
                               train_params="{\"arg1\":1, \"arg2\":\"test\"}")
        result = train_task.save()
        self.assertTrue(result)
        self.assertEqual(train_task.id, "TRAI001")

    def test_create_train_task_ko_empty_name_projectid_algoid_trainparams(self):
        train_task = TrainTask()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            train_task.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "TrainTask缺少必要字段：['name', 'project_id', 'algorithm_id', 'train_params']")

    @responses.activate
    def test_create_train_task_ko_unknown_response(self):
        responses.add(responses.POST, url=self._url("train_task/add"),
                      json={'msg': "Unknown response"}, status=201)
        train_task = TrainTask(name="TestTrainTask250", project_id="PROJ250",
                               algorithm_id="ALGO250",
                               train_params="{\"arg1\":1, \"arg2\":\"test\"}")
        with self.assertRaises(AnyLearnException) as ctx:
            train_task.save()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_delete_train_task_ok(self):
        responses.add(responses.DELETE,
                      url=self._url("train_task/delete?id=TRAI001&force=0"),
                      match_querystring=True,
                      json={'data': "TRAI001", 'message': "任务删除成功"},
                      status=200)
        train_task = TrainTask(id="TRAI001")
        result = train_task.delete()
        self.assertTrue(result)

    @responses.activate
    def test_delete_train_task_ko_unknown_response(self):
        responses.add(responses.DELETE,
                      url=self._url("train_task/delete?id=TRAI250&force=0"),
                      match_querystring=True,
                      json={'msg': "Unknown response"},
                      status=204)
        train_task = TrainTask(id="TRAI250")
        with self.assertRaises(AnyLearnException) as ctx:
            train_task.delete()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_get_train_task_log_ok(self):
        query_str = urlencode({
            'id': "TRAI001",
            'limit': 100,
            'direction': "init",
            'index': 0,
            'offset_index': -1,
        })
        responses.add(responses.GET,
                      url=self._url(f"train_task/log?{query_str}"),
                      match_querystring=True,
                      json=[
                          {'offset': 1, 'offset_index': 1, 'text': "log1"},
                          {'offset': 2, 'offset_index': 2, 'text': "log2"},
                          {'offset': 3, 'offset_index': 3, 'text': "log3"},
                      ],
                      status=200)
        train_task = TrainTask(id="TRAI001")
        result = train_task.get_log()
        self.assertIsInstance(result, list)
        self.assertEqual(len(result), 3)
        self.assertIn('text', result[0])
        self.assertIn('text', result[1])
        self.assertIn('text', result[2])

    def test_get_service_log_ko_empty_id(self):
        train_task = TrainTask(name="TestTrainTask")
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            train_task.get_log()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "TrainTask缺少必要字段：['id']")

    @responses.activate
    def test_get_train_task_log_ko_unknown_response(self):
        query_str = urlencode({
            'id': "TRAI250",
            'limit': 100,
            'direction': "init",
            'index': 0,
            'offset_index': -1,
        })
        responses.add(responses.GET,
                      url=self._url(f"train_task/log?{query_str}"),
                      match_querystring=True,
                      json="",
                      status=204)
        train_task = TrainTask(id="TRAI250")
        with self.assertRaises(AnyLearnException) as ctx:
            train_task.get_log()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_get_train_task_last_log_ok(self):
        query_str = urlencode({
            'id': "TRAI001",
            'limit': 100,
            'direction': "init",
            'index': 0,
            'offset_index': -1,
        })
        responses.add(responses.GET,
                      url=self._url(f"train_task/log?{query_str}"),
                      match_querystring=True,
                      json=[
                          {'offset': 3, 'offset_index': 3, 'text': "log3"},
                          {'offset': 2, 'offset_index': 2, 'text': "log2"},
                          {'offset': 1, 'offset_index': 1, 'text': "log1"},
                      ],
                      status=200)
        train_task = TrainTask(id="TRAI001")
        result = train_task.get_last_log(debug=False)
        tz = datetime.now().astimezone().tzinfo
        self.assertIsInstance(result, list)
        self.assertEqual(len(result), 3)
        self.assertEqual(result[0], f"[{str(datetime.fromtimestamp(1/1000, tz=tz))}] log1")
        self.assertEqual(result[1], f"[{str(datetime.fromtimestamp(2/1000, tz=tz))}] log2")
        self.assertEqual(result[2], f"[{str(datetime.fromtimestamp(3/1000, tz=tz))}] log3")

    @responses.activate
    def test_get_train_task_full_log_ok(self):
        log_res = [
            {'offset': i, 'offset_index': i, 'text': f"log{i}"}
            for i in range(200)
        ]
        query_str_1 = urlencode({
            'id': "TRAI001",
            'limit': 100,
            'direction': "back",
            'index': 0,
            'offset_index': -1,
        })
        query_str_2 = urlencode({
            'id': "TRAI001",
            'limit': 100,
            'direction': "back",
            'index': 99,
            'offset_index': 99,
        })
        responses.add(responses.GET,
                      url=self._url(f"train_task/log?{query_str_1}"),
                      match_querystring=True,
                      json=log_res[:100],
                      status=200)
        responses.add(responses.GET,
                      url=self._url(f"train_task/log?{query_str_2}"),
                      match_querystring=True,
                      json=log_res[100:],
                      status=200)
        train_task = TrainTask(id="TRAI001")
        result = train_task.get_full_log(debug=False)
        tz = datetime.now().astimezone().tzinfo
        self.assertIsInstance(result, list)
        self.assertEqual(len(result), 200)
        self.assertEqual(result[0], f"[{str(datetime.fromtimestamp(0, tz=tz))}] log0")
        self.assertEqual(result[99], f"[{str(datetime.fromtimestamp(99/1000, tz=tz))}] log99")
        self.assertEqual(result[199], f"[{str(datetime.fromtimestamp(199/1000, tz=tz))}] log199")

    @responses.activate
    def test_get_train_task_status_ok(self):
        responses.add(responses.GET, url=self._url("train_task/status?id=TRAI001&secret_key=SECRET"),
                      match_querystring=True, json={
                          'current_epoch': "6",
                          'current_train_loss': "1.234",
                          'current_train_step': "567",
                          'ip': "10.244.2.165",
                          'process': "0.987",
                          'secret_key': "SECRET",
                          'state': "working",
                      },
                      status=200)
        train_task = TrainTask(id="TRAI001", secret_key="SECRET")
        res = train_task.get_status()
        self.assertIsInstance(res, dict)
        self.assertIn('current_epoch', res)
        self.assertIn('current_train_loss', res)
        self.assertIn('current_train_step', res)
        self.assertIn('process', res)
        self.assertIn('ip', res)
        self.assertIn('secret_key', res)

    def test_get_train_task_status_ko_empty_id_secretkey(self):
        train_task = TrainTask()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            train_task.get_status()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "TrainTask缺少必要字段：['id', 'secret_key']")

    @responses.activate
    def test_get_train_task_status_ko_unknown_response(self):
        responses.add(responses.GET,
                      url=self._url("train_task/status?id=TRAI250&secret_key=SECRET250"),
                      match_querystring=True,
                      json=[{'msg': "Unknown response"}],
                      status=204)
        train_task = TrainTask(id="TRAI250", secret_key="SECRET250")
        with self.assertRaises(AnyLearnException) as ctx:
            train_task.get_status()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_report_final_metric_ok(self):
        responses.add(responses.POST, url=self._url("train_task/final_metric"),
                      json={
                          'msg': "任务TRAId123结果指标保存成功",
                      },
                      status=200)
        train_task = TrainTask(id="TRAI123", secret_key="SECRET")
        res = train_task.report_final_metric(metric=66.8)
        self.assertIsInstance(res, dict)
        self.assertEqual("任务TRAId123结果指标保存成功", res['msg'])
        self.assertEqual(train_task.final_metric, 66.8)

    def test_report_final_metric_empty_id_secretkey(self):
        train_task = TrainTask()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            train_task.report_final_metric(250.41)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "TrainTask缺少必要字段：['id', 'secret_key']")

    @responses.activate
    def test_report_final_metric_unknown_response(self):
        responses.add(responses.POST,
                      url=self._url("train_task/final_metric"),
                      json=[{'msg': "Unknown response"}],
                      status=204)
        train_task = TrainTask(id="TRAI250", secret_key="SECRET")
        with self.assertRaises(AnyLearnException) as ctx:
            train_task.report_final_metric(250.41)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_get_final_metric_ok(self):
        responses.add(responses.GET,
                      url=self._url("train_task/final_metric?id=TRAI001"),
                      match_querystring=True,
                      json={
                          'final_metric': 662.8,
                          'id': "TRAI001",
                          'name': "test"
                      },
                      status=200)
        train_task = TrainTask(id="TRAI001")
        res = train_task.get_final_metric()
        self.assertIsInstance(res, dict)
        self.assertIn('id', res)
        self.assertIn('name', res)
        self.assertIn('final_metric', res)

    def test_get_final_metric_empty_id(self):
        train_task = TrainTask()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            train_task.get_final_metric()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "TrainTask缺少必要字段：['id']")

    @responses.activate
    def test_get_final_metric_unknown_response(self):
        responses.add(responses.GET,
                      url=self._url("train_task/final_metric?id=TRAI250"),
                      match_querystring=True,
                      json=[{'msg': "Unknown response"}],
                      status=204)
        train_task = TrainTask(id="TRAI250", secret_key="SECRET250")
        with self.assertRaises(AnyLearnException) as ctx:
            train_task.get_final_metric()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_report_intermediate_metric_ok(self):
        responses.add(responses.POST,
                      url=self._url("train_task/intermediate_metric"),
                      json={
                          'msg': "任务TRAId123结果指标保存成功",
                      },
                      status=200)
        train_task = TrainTask(id="TRAI123", secret_key="SECRET")
        res = train_task.report_intermediate_metric(88.6)
        self.assertIsInstance(res, dict)
        self.assertEqual("任务TRAId123结果指标保存成功", res['msg'])

    def test_report_intermediate_metric_empty_id_secretkey(self):
        train_task = TrainTask()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            train_task.report_intermediate_metric(88.8)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "TrainTask缺少必要字段：['id', 'secret_key']")

    @responses.activate
    def test_report_intermediate_metric_unknown_response(self):
        responses.add(responses.POST,
                      url=self._url("train_task/intermediate_metric"),
                      json=[{'msg': "Unknown response"}],
                      status=204)
        train_task = TrainTask(id="TRAI250", secret_key="SECRET")
        with self.assertRaises(AnyLearnException) as ctx:
            train_task.report_intermediate_metric(88.8)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @responses.activate
    def test_get_intermediate_metric_ok(self):
        dt = "2021-05-19 00:00:00"
        query_str = urlencode({
            'id': "TRAI123",
            'last_timestamp': dt,
        })
        responses.add(responses.GET,
                      url=self._url(f"train_task/intermediate_metric?{query_str}"),
                      match_querystring=True,
                      json=[
                          {
                              "id": "METR123",
                              "metric": 90.0,
                              "train_task_id": "TRAI123",
                          },
                          {
                              "id": "METR456",
                              "metric": 90.2,
                              "train_task_id": "TRAI123",
                          },
                      ],
                      status=200)
        train_task = TrainTask(id="TRAI123")
        res = train_task.get_intermediate_metric(last_timestamp=dt)
        self.assertIsInstance(res, list)
        self.assertEqual(len(res), 2)
        self.assertIn('id', res[0])
        self.assertIn('id', res[1])
        self.assertIn('metric', res[0])
        self.assertIn('metric', res[1])
        self.assertIn('train_task_id', res[0])
        self.assertIn('train_task_id', res[1])

    def test_get_intermediate_metric_empty_id(self):
        train_task = TrainTask()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            train_task.get_intermediate_metric()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "TrainTask缺少必要字段：['id']")

    @responses.activate
    def test_get_intermediate_metric_unknown_response(self):
        dt = "2021-05-19 00:00:00"
        query_str = urlencode({
            'id': "TRAI250",
            'last_timestamp': dt,
        })
        responses.add(responses.GET,
                      url=self._url(f"train_task/intermediate_metric?{query_str}"),
                      match_querystring=True,
                      json={'msg': "Unknown response"},
                      status=204)
        train_task = TrainTask(id="TRAI250")
        with self.assertRaises(AnyLearnException) as ctx:
            train_task.get_intermediate_metric(last_timestamp=dt)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    @patch('anylearn.interfaces.resource.resource_downloader.ResourceDownloader')
    @responses.activate
    def test_download_results_ok(self, MockResourceDownloader):
        @create_autospec
        def mock_download_ok(resource_id, save_path, polling):
            return True

        downloader = MockResourceDownloader()
        downloader.run = mock_download_ok

        save_path = "tests/"
        train_task = TrainTask(results_id="FILE001",
                               state=TrainTaskState.SUCCESS)
        res = train_task.download_results(
            save_path=save_path, downloader=downloader)
        self.assertTrue(res)

    @responses.activate
    def test_download_results_state_not_success(self):

        train_task = TrainTask(results_id="FILE001",
                               state=TrainTaskState.CREATED)
        with self.assertRaises(AnyLearnException) as ctx:
            save_path = "tests/"
            train_task.download_results(
                save_path=save_path, downloader=None)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "训练未开始!")

    @responses.activate
    def test_download_results_state_empty_results_id(self):

        train_task = TrainTask()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            save_path = "tests/"
            train_task.download_results(
                save_path=save_path, downloader=None)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(e.msg, "TrainTask缺少必要字段：['results_id']")

    @responses.activate
    def test_transform_model_ok(self):
        responses.add(responses.POST,
                      url=self._url("model/transform"),
                      match_querystring=True,
                      json={
                          "data": "MODE001",
                          "message": "模型转存工作正在进行，请稍后查看"
                      },
                      status=200)

        responses.add(responses.GET, url=self._url("model/query?id=MODE001"),
                      match_querystring=True, json=[{
                          'id': "MODE001",
                          'name': "TestModel1",
                          'description': "test",
                          'state': ResourceState.READY,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "test.tar.gz",
                          'is_zipfile': 1,
                          'file': "USER001/models/MODE001/test.tar.gz",
                          'size': 250.41,
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                          'owner': ["USER001"],
                          'algorithm_id': "ALGO001",
                      }],
                      status=200)
        train_task = TrainTask(results_id="FILE123", algorithm_id="ALGO123")
        owner = ["USER1", "USER2", "USER13"]
        res = train_task.transform_model(
            "file_path", "name", "description", True, False, 1)
        self.assertIsInstance(res, Model)
        self.assertEqual(res.id, "MODE001")
        self.assertEqual(res.state, ResourceState.READY)

    @responses.activate
    def test_transform_model_error(self):
        responses.add(responses.POST,
                      url=self._url("model/transform"),
                      match_querystring=True,
                      json={
                          "data": "MODE001",
                          "message": "模型转存工作正在进行，请稍后查看"
                      },
                      status=200)

        responses.add(responses.GET, url=self._url("model/query?id=MODE001"),
                      match_querystring=True, json=[{
                          'id': "MODE001",
                          'name': "TestModel1",
                          'description': "test",
                          'state': ResourceState.ERROR,
                          'public': True,
                          'upload_time': "2020-01-01 00:00",
                          'filename': "test.tar.gz",
                          'is_zipfile': 1,
                          'file': "USER001/models/MODE001/test.tar.gz",
                          'size': 250.41,
                          'creator_id': "USER001",
                          'node_id': "NODE001",
                          'owner': ["USER001"],
                          'algorithm_id': "ALGO001",
                      }],
                      status=200)
        train_task = TrainTask(results_id="FILE123", algorithm_id="ALGO123")
        with self.assertRaises(AnyLearnException) as ctx:
            owner = ["USER1", "USER2", "USER13"]
            res = train_task.transform_model(
                "file_path", "name", "description", True, False, 1)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "Error occured when transforming model")

    def test_transform_model_empty_results_id_algorithm_id(self):
        train_task = TrainTask()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            owner = ["USER1", "USER2", "USER13"]
            train_task.transform_model(
                "file_path", "name", "description", True, False, 1)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(
            e.msg, "TrainTask缺少必要字段：['results_id', 'algorithm_id']")

    @responses.activate
    def test_transform_model_unknown_response(self):
        responses.add(responses.POST,
                      url=self._url("model/transform"),
                      match_querystring=True,
                      json=[{'msg': "Unknown response"}],
                      status=204)
        train_task = TrainTask(results_id="FILE123", algorithm_id="ALGO123")
        with self.assertRaises(AnyLearnException) as ctx:
            owner = ["USER1", "USER2", "USER13"]
            train_task.transform_model(
                "file_path", "name", "description", True, False, 1)
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, "请求未能得到有效响应")

    def test_get_train_profile_ok(self):
        test_train_task = TrainTask(id="TRAI_test_get_train_profile",
                                    state=2,
                                    secret_key="test_key",
                                    project_id="test_project_id",)
        DB().create_or_update_train_task(train_task=test_train_task)
        train_profile = TrainProfile(id=uuid.uuid1().__str__(),
                                     train_task_id=test_train_task.id,
                                     entrypoint="train_task.entrypoint",
                                     algorithm_id="algo.id",
                                     dataset_id="dset.id",
                                     train_params="train_task.train_params",
                                     algorithm_dir="algorithm_dir",
                                     algorithm_archive="algorithm_archive",
                                     dataset_dir="dataset_dir",
                                     dataset_archive="dataset_archive",)
        train_profile.create_in_db()
        res = test_train_task.get_train_profile()
        if res:
            self.assertIsInstance(res, TrainProfile)
            self.assertEqual("TRAI_test_get_train_profile", res.train_task_id)

    def test_get_train_profile_id_not_found(self):
        test_train_task = TrainTask(id="abcdefg~^%*#@")
        with self.assertRaises(AnyLearnException) as ctx:
            test_train_task.get_train_profile()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnException)
        self.assertEqual(e.msg, f"资源{test_train_task.id}未找到")

    def test_transform_model_empty_id(self):
        test_train_task = TrainTask()
        with self.assertRaises(AnyLearnMissingParamException) as ctx:
            test_train_task.get_train_profile()
        e = ctx.exception
        self.assertIsInstance(e, AnyLearnMissingParamException)
        self.assertEqual(
            e.msg, "TrainTask缺少必要字段：['id']")



import click

from anylearn.cli.anylearn_cli_config import AnylearnCliConfig
from anylearn.cli.utils import (
    check_connection,
    cmd_confirm_or_abort,
    option_force,
    check_config,
    cmd_error,
    cmd_info,
    cmd_success,
    cmd_warning,
    get_cmd_command,
)
from anylearn.interfaces import Project, TrainTask


@click.group("view")
def commands():
    """
    Show Web UI on specific objects.
    """
    pass


@check_config()
@check_connection()
def __host__():
    config = AnylearnCliConfig.load()
    return config.remote['host']


@commands.command()
@get_cmd_command()
def ui():
    click.launch(__host__())


@commands.command()
@click.argument('project_id')
@get_cmd_command()
def project(project_id):
    host = __host__()
    try:
        Project(id=project_id, load_detail=True)
        click.launch(f"{host}/project/{project_id}/detail")
    except:
        cmd_error(msg=f"Failed to fetch project {project_id}.")
        raise click.Abort


@commands.command()
@click.argument('task_id')
@get_cmd_command()
def task(task_id: str):
    host = __host__()
    task = TrainTask(id=task_id, load_detail=True)
    click.launch(f"{host}/project/{task.project_id}/traintask/{task.id}/detail")



import json

import click

from anylearn.cli.anylearn_cli_config import AnylearnCliConfig
from anylearn.cli.utils import (
    cmd_confirm_or_abort,
    check_config,
    check_connection,
    cmd_error,
    cmd_info,
    cmd_success,
    cmd_warning,
    get_cmd_command,
)
from anylearn.interfaces import TrainTask


@click.group("task")
def commands():
    """
    Show tasks created within local Anylearn project.
    """
    pass


@commands.command()
@check_config()
@check_connection()
@get_cmd_command()
def ls():
    """
    List all tasks.
    """
    config = AnylearnCliConfig.load()
    if not config.project or not config.project.id:
        cmd_error(msg=(
            "Local project is broken or has not been published to Anylearn remote.\n"
            "Call `anyctl reset` then `anyctl init` if project configuration is corrupted, "
            f"or call `anyctl push project` to publish local project."
        ))
        raise click.Abort
    tasks = config.project.get_train_tasks(load_detail=True)
    algos = []
    for t in tasks:
        try:
            algos.append(next(
                k
                for k, v in config.algorithms.items()
                if v.id == t.algorithm_id
            ))
        except StopIteration:
            algos.append(t.algorithm_id)
    msgs = [
        f"{click.style(t.id, fg='yellow')} <{t.create_time}> <{a}>"
        for t, a in zip(tasks, algos)
    ]
    cmd_info(msg="\n".join(msgs))


@commands.command()
@click.argument('task_id')
@check_config()
@check_connection()
@get_cmd_command()
def get(task_id: str):
    """
    Get task detail.
    """
    config = AnylearnCliConfig.load()
    try:
        task = TrainTask(id=task_id, load_detail=True)
    except:
        cmd_error(msg=f"Failed to get task {task_id}")
        raise click.Abort
    states = {
        '-3': 'Aborted',
        '-2': 'Failed',
        '-1': 'Deleted',
        '0': 'Created',
        '1': 'Running',
        '2': 'Succeeded',
        '3': 'Pending',
        '4': 'Retrying',
        '5': 'Locked',
    }
    cmd_info(msg=(
        f"ID : {task.id}\n"
        f"Name : {task.name}\n"
        f"Description : {task.description}\n"
        f"State : {states.get(str(task.state), 'Unknown')}\n"
        f"Algorithm ID : {task.algorithm_id}\n"
        f"hyperparameters : {task.train_params}\n"
        f"datasets/models : {task.files}\n"
        f"create_time : {task.create_time}\n"
        f"finish_time : {task.finish_time}\n"
        f"final_metric : {task.final_metric}\n"
    ))
